@@ -6,6 +6,7 @@
 #'
 #' @param x A `finbif_records*` object.
 #' @param ... Additional arguments. Not used.
+#' @param locale Character. A locale to use for columns with localised data.
 #' @param quiet Logical. If `TRUE` (default) suppress progress indicator of
 #'   conversion.
 #' @return A `data.frame`.
@@ -19,90 +20,111 @@
 #' @importFrom methods as
 #' @importFrom utils txtProgressBar setTxtProgressBar
 #' @export
-as.data.frame.finbif_records <- function(x, ...) {
+as.data.frame.finbif_records <-
+  function(x,  ..., locale = getOption("finbif_locale")) {
 
-  cols <- attr(x, "select")
-  url  <- x[["response"]][["url"]]
-  time <- x[["response"]][["date"]]
+    cols <- attr(x, "select")
+    url  <- x[["response"]][["url"]]
+    time <- x[["response"]][["date"]]
 
-  x <- x[["content"]][["results"]]
+    x <- x[["content"]][["results"]]
 
-  lst <- lapply(
-    cols,
-    function(col) {
-      type <- var_names[col, "type"]
-      type_na <- methods::as(NA, type)
-      single <- var_names[col, "single"]
-      el_names <- strsplit(col, "\\.")[[1L]]
-      if (single) return(vapply(x, get_el_recurse, type_na, el_names, type))
-      ans <- lapply(x, get_el_recurse, el_names, type)
-      lapply(ans, unlist)
-    }
-  )
+    lst <- lapply(
+      cols,
+      function(col) {
+        type      <- var_names[col, "type"]
+        type_na   <- methods::as(NA, type)
+        single    <- var_names[col, "single"]
+        localised <- var_names[col, "localised"]
+        el_names <- strsplit(col, "\\.")[[1L]]
+        if (single) return(vapply(x, get_el_recurse, type_na, el_names, type))
+        ans <- lapply(x, get_el_recurse, el_names, type)
+        ans <- lapply(ans, unlist)
+        if (localised) ans <- vapply(ans, with_locale, type_na, locale)
+        ans
+      }
+    )
 
-  names(lst) <- cols
-  cols_split <- split(cols, var_names[cols, "single"])
-  cols_split[["TRUE"]] <- c("ind", cols_split[["TRUE"]])
-  lst <- c(list(ind = seq_along(x)), lst)
-  df <- as.data.frame(lst[cols_split[["TRUE"]]], stringsAsFactors = FALSE)
-  df[cols_split[["FALSE"]]] <- lst[cols_split[["FALSE"]]]
-  df[["ind"]] <- NULL
-  structure(df[cols], url = url, time = time)
+    names(lst) <- cols
+    cols_split <- split(cols, var_names[cols, "single"])
+    cols_split[["TRUE"]] <- c("ind", cols_split[["TRUE"]])
+    lst <- c(list(ind = seq_along(x)), lst)
+    df <- as.data.frame(lst[cols_split[["TRUE"]]], stringsAsFactors = FALSE)
+    df[cols_split[["FALSE"]]] <- lst[cols_split[["FALSE"]]]
+    df[["ind"]] <- NULL
+    structure(df[cols], url = url, time = time)
 
+  }
+
+#' @noRd
+with_locale <- function(x, locale) {
+  if (identical(length(x), 1L)) return(x[[1L]])
+  x[[intersect(c(locale, setdiff(supported_langs, locale)), names(x))[[1L]]]]
 }
 
 #' @rdname as.data.frame.finbif_records
 #' @export
-as.data.frame.finbif_records_list <- function(x, ..., quiet = TRUE) {
+as.data.frame.finbif_records_list <-
+  function(x, ..., locale = getOption("finbif_locale"), quiet = TRUE) {
 
-  n <- length(x)
-  if (!quiet) {
-    pb <- utils::txtProgressBar(0L, n, style = 3L)
-    on.exit(close(pb))
-  }
-
-  df <- lapply(
-    seq_len(n),
-    function(i) {
-      if (!quiet) utils::setTxtProgressBar(pb, i)
-      as.data.frame(x[[i]])
+    n <- length(x)
+    if (!quiet) {
+      pb <- utils::txtProgressBar(0L, n, style = 3L)
+      on.exit(close(pb))
     }
-  )
 
-  url  <- do.call(c, lapply(df, attr, "url", TRUE))
-  time <- do.call(c, lapply(df, attr, "time", TRUE))
+    df <- lapply(
+      seq_len(n),
+      function(i) {
+        if (!quiet) utils::setTxtProgressBar(pb, i)
+        as.data.frame(x[[i]], locale = locale)
+      }
+    )
 
-  df <- do.call(rbind, df)
+    url  <- do.call(c, lapply(df, attr, "url", TRUE))
+    time <- do.call(c, lapply(df, attr, "time", TRUE))
 
-  if (inherits(x, "finbif_records_sample_list")) {
-    records <- if (attr(x, "cache")) {
-      sample_with_seed(nrow(df), nrow(df), gen_seed(x))
-    } else {
-      sample.int(nrow(df))
+    df <- do.call(rbind, df)
+
+    if (inherits(x, "finbif_records_sample_list")) {
+      records <- if (attr(x, "cache")) {
+        sample_with_seed(nrow(df), nrow(df), gen_seed(x))
+      } else {
+        sample.int(nrow(df))
+      }
+      df <- df[records, ]
     }
-    df <- df[records, ]
+
+    if (!attr(x, "record_id")) df[["unit.unitId"]] <- NULL
+
+    structure(df, url = url, time = time)
+
   }
 
-  if (!attr(x, "record_id")) df[["unit.unitId"]] <- NULL
-
-  structure(df, url = url, time = time)
-
-}
-
 # accessor methods -------------------------------------------------------------
 
 #' @noRd
 #' @export
-`[.finbif_occ` <- function(x, i, j, drop = TRUE) {
-  if (missing(j)) {
-    cols <- i
-    rows <- seq_len(nrow(x))
+`[.finbif_occ` <- function(x, i, j, drop = FALSE) {
+
+  mdrop  <- missing(drop)
+  n_args <- nargs() - !mdrop
+  has_i  <- !missing(i)
+  has_j  <- !missing(j)
+
+  rows <- seq_len(nrow(x))
+  cols <- seq_len(ncol(x))
+
+  if (n_args < 3L) {
+    if (has_i) cols <- i
   } else {
-    cols <- j
-    rows <- i
+    if (has_j) cols <- j
+    if (has_i) rows <- i
   }
-  ans <- NextMethod("[")
-  ans <- structure(ans, class = class(x))
+
+  ans <- if (has_j) NextMethod("[", drop = drop) else NextMethod("[")
+  ans <-
+    structure(as.data.frame(ans, stringsAsFactors = FALSE), class = class(x))
   attr <- attributes(x)
   attr[["row.names"]] <- as.integer(rows)
   mostattributes(ans) <- attr
@@ -175,47 +197,42 @@ print.finbif_metadata_df <- function(x, ..., right = FALSE) {
 #' @importFrom utils hasName head
 #' @export
 print.finbif_occ <- function(x, ...) {
+
   nrec_dnld <- attr(x, "nrec_dnld")
   nrec_avl  <- attr(x, "nrec_avl")
   ncols     <- ncol(x)
   nrows     <- nrow(x)
   dsply_nr  <- min(10L, nrows)
-  dsply_nc  <- min(5L, ncols)
 
   if (length(nrec_dnld)) cat("Records downloaded: ", nrec_dnld, "\n", sep = "")
   if (length(nrec_avl)) cat("Records available: ", nrec_avl, "\n", sep = "")
   cat("A data.frame [", nrows, " x ", ncols, "]\n", sep = "")
 
-  dwc <- attr(x, "dwc")
+  df <- x[seq_len(dsply_nr), , drop = FALSE]
 
-  dsply_cols <- if (dwc) {
-    c(
-      "scientificName", "individualCount", "decimalLatitude",
-      "decimalLongitude", "eventDateTime"
-    )
-  } else {
-    c("scientific_name", "abundance", "lat_wgs84", "lon_wgs84", "date_time")
+  colname_widths <- vapply(names(df), nchar, integer(1L))
+
+  df <- format_cols(df, colname_widths)
+
+  widths <- apply(df, 2L, nchar)
+  # Printed NA values are four characters wide, "<NA>"
+  widths[is.na(widths)] <- 4L
+  widths <- apply(widths, 2L, max, na.rm = TRUE)
+  widths <- pmax(colname_widths, widths)
+
+  dsply_nc <- 0L
+  cumulative_width <- if (dsply_nr > 9L) 2L else 1L
+
+  for (i in widths) {
+    ind <- dsply_nc + 1L
+    cumulative_width <- cumulative_width + 1L + widths[[ind]]
+    if (cumulative_width > getOption("width")) break
+    dsply_nc <- ind
   }
-  dsply_cols <- which(names(x) %in% dsply_cols)
-  dsply_cols <- utils::head(union(dsply_cols, seq_len(dsply_nc)), dsply_nc)
 
-  df <- x[seq_len(dsply_nr), dsply_cols, drop = FALSE]
+  dsply_cols <- seq_len(dsply_nc)
 
-  # Some scientific names are very long
-  if (utils::hasName(df, "scientific_name"))
-    df[["scientific_name"]] <- truncate_string(df[["scientific_name"]])
-  if (utils::hasName(df, "scientificName"))
-    df[["scientificName"]] <- truncate_string(df[["scientificName"]])
-
-  # Some vars have data in the form of URIs where the protocol and domain don't
-  # convey useful information
-  for (i in names(df)) {
-    class <-
-      var_names[var_names[[if (dwc) "dwc" else "translated_var"]] == i, "class"]
-    # Variables may not necessarily be in the var_names object
-    if (length(class) && class == "uri")
-      df[[i]] <- gsub("^http:\\/\\/tun\\.fi\\/", "", df[[i]])
-  }
+  df <- df[, dsply_cols]
 
   print.data.frame(df)
 
@@ -225,6 +242,43 @@ print.finbif_occ <- function(x, ...) {
   print_extras(x, extra_rows, extra_cols, dsply_cols)
 
   invisible(x)
+
+}
+
+#' @noRd
+format_cols <- function(df, colname_widths) {
+
+  for (i in names(df)) {
+
+    ind <- var_names[[if (attr(df, "dwc")) "dwc" else "translated_var"]] == i
+    class  <- var_names[ind, "class"]
+    single <- var_names[ind, "single"] || var_names[ind, "localised"]
+
+    # Variables may not necessarily be in the var_names object
+    if (length(class))
+      if (!single) {
+        df[[i]] <- vapply(
+          df[[i]],
+          function(x) length(Filter(isFALSE, is.na(x))), integer(1L)
+        )
+        df[[i]] <-
+          paste0(df[[i]], " element", ifelse(length(df[[i]]) == 1L, "", "s"))
+      } else {
+        if (class == "uri") df[[i]] <- truncate_string_to_unique(df[[i]])
+        if (class %in% c("character", "uri", "factor"))
+          df[[i]] <- truncate_string(df[[i]])
+        if (class %in% c("double", "integer"))
+          df[[i]] <- formatC(
+            df[[i]],
+            colname_widths[[i]] - 2L, colname_widths[[i]],
+            flag = "- "
+          )
+      }
+
+  }
+
+  df
+
 }
 
 #' @noRd
@@ -256,7 +310,7 @@ print_extras <- function(x, extra_rows, extra_cols, dsply_cols) {
   for (i in seq_along(extra_names)[-1L]) {
     nchars_ <- nchar(extra_names[[i]]) + 2L
     nchars  <- nchars + nchars_
-    if (nchars > 70L) {
+    if (nchars > getOption("width")) {
       cat(",\n")
       nchars <- nchars_
     } else {
