@@ -3,7 +3,7 @@ context("Checking clearing FinBIF cache")
 test_that(
   "works", {
 
-    expect_identical(finbif_clear_cache(), 0L)
+    expect_null(finbif_clear_cache())
 
   }
 )
